@@ -1,6 +1,6 @@
-# Copyright (c) 2010 Infrae. All rights reserved.
+# -*- coding: utf-8 -*-
+# Copyright (c) 2010-2012 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id$
 
 from cgi import escape
 from urllib import quote
@@ -9,12 +9,12 @@ import re
 
 from ZPublisher.HTTPResponse import status_reasons
 from ZPublisher.Iterators import IStreamIterator
-from ZPublisher.pubevents import PubBeforeStreaming
 from zope.publisher.interfaces.http import IResult
 from zope.event import notify
 import zExceptions
 
 from infrae.wsgi.headers import HTTPHeaders
+from infrae.wsgi.interfaces import PublicationBeforeStreaming
 from infrae.wsgi.log import log_invalid_response_data
 
 HEAD_REGEXP = re.compile('(<head[^>]*>)', re.I)
@@ -151,6 +151,9 @@ class WSGIResponse(object):
                 body = body.encode(self.default_charset)
         self.body = body
 
+    def getBody(self):
+        return self.body
+
     def setStatus(self, status, msg=None):
         # We ignore msg and use our own.
         self.status = status
@@ -235,7 +238,7 @@ class WSGIResponse(object):
                         self.default_charset,)
         else:
             # Fire event before streaming
-            notify(PubBeforeStreaming(self))
+            notify(PublicationBeforeStreaming(self))
 
             # Fix default Content-Type
             if not self.headers.has_key('Content-Type'):
@@ -276,6 +279,7 @@ class WSGIResponse(object):
         raise zExceptions.Unauthorized("Please authenticate")
 
     def debugError(self, name):
+        import pdb; pdb.set_trace()
         raise zExceptions.NotFound(name)
 
     def forbiddenError(self, name):
