@@ -1,6 +1,6 @@
-# Copyright (c) 2010 Infrae. All rights reserved.
+# -*- coding: utf-8 -*-
+# Copyright (c) 2010-2012 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id$
 
 from cStringIO import StringIO
 import unittest
@@ -24,6 +24,8 @@ from infrae.wsgi.tests.mockers import (
 
 DEFAULT_ENVIRON = {
     'wsgi.url_scheme': 'http',
+    'SERVER_NAME': 'infrae.com',
+    'SERVER_PORT': '80',
     'HTTP_HOST': 'infrae.com',
     'PATH_INFO': '/index.html'}
 
@@ -232,7 +234,7 @@ class PublisherTestCase(unittest.TestCase):
         request = self.new_request_for(hello_view)
         with LoggingTesting('infrae.wsgi') as logs:
             publication = WSGIPublication(self.app, request, self.response)
-            result = publication()
+            result = publication(lambda: request.close())
 
             self.assertEqual(
                 request.mocker_called(),
@@ -250,7 +252,11 @@ class PublisherTestCase(unittest.TestCase):
                  ('Content-Type', 'text/html;charset=utf-8')])
             self.assertEqual(
                 get_event_names(),
-                ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+                ['PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationAfterRender',
+                 'PublicationBeforeCommit',
+                 'PublicationSuccess'])
 
             body = consume_wsgi_result(result)
 
@@ -270,7 +276,7 @@ class PublisherTestCase(unittest.TestCase):
         request = self.new_request_for(no_content_view)
         with LoggingTesting('infrae.wsgi') as logs:
             publication = WSGIPublication(self.app, request, self.response)
-            result = publication()
+            result = publication(lambda: request.close())
 
             self.assertEqual(
                 request.mocker_called(),
@@ -287,7 +293,11 @@ class PublisherTestCase(unittest.TestCase):
                 [('Content-Length', '0')])
             self.assertEqual(
                 get_event_names(),
-                ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+                ['PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationAfterRender',
+                 'PublicationBeforeCommit',
+                 'PublicationSuccess'])
 
             body = consume_wsgi_result(result)
 
@@ -309,7 +319,8 @@ class PublisherTestCase(unittest.TestCase):
         request = self.new_request_for(result_view)
         with LoggingTesting('infrae.wsgi') as logs:
             publication = WSGIPublication(self.app, request, self.response)
-            result = publication()
+            result = publication(lambda: request.close())
+
 
             self.assertEqual(
                 request.mocker_called(),
@@ -325,7 +336,10 @@ class PublisherTestCase(unittest.TestCase):
                 [('Content-Type', 'text/html;charset=utf-8')])
             self.assertEqual(
                 get_event_names(),
-                ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])
+                ['PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationAfterRender',
+                 'PublicationBeforeStreaming'])
 
             body = consume_wsgi_result(result)
 
@@ -336,8 +350,11 @@ class PublisherTestCase(unittest.TestCase):
                 self.app.transaction.mocker_called(), [('commit', (), {})])
             self.assertEqual(
                 get_event_names(),
-                ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
-                 'PubBeforeCommit', 'PubSuccess'])
+                ['TestNextCalled',
+                 'TestNextCalled',
+                 'TestNextCalled',
+                 'PublicationBeforeCommit',
+                 'PublicationSuccess'])
 
             logs.assertEmpty()
 
@@ -349,7 +366,7 @@ class PublisherTestCase(unittest.TestCase):
         request = self.new_request_for(bugous_result_view)
         with LoggingTesting('infrae.wsgi') as logs:
             publication = WSGIPublication(self.app, request, self.response)
-            result = publication()
+            result = publication(lambda: request.close())
 
             self.assertEqual(
                 request.mocker_called(),
@@ -365,7 +382,10 @@ class PublisherTestCase(unittest.TestCase):
                 [('Content-Type', 'text/html;charset=utf-8')])
             self.assertEqual(
                 get_event_names(),
-                ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])
+                ['PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationAfterRender',
+                 'PublicationBeforeStreaming'])
 
             self.assertRaises(ValueError, consume_wsgi_result, result)
 
@@ -375,7 +395,9 @@ class PublisherTestCase(unittest.TestCase):
                 self.app.transaction.mocker_called(), [('abort', (), {})])
             self.assertEqual(
                 get_event_names(),
-                ['TestNextCalled', 'PubBeforeAbort', 'PubFailure'])
+                ['TestNextCalled',
+                 'PublicationBeforeAbort',
+                 'PublicationFailure'])
 
             logs.assertContains(
                 "An error happened in the WSGI stack while iterating "
@@ -391,7 +413,7 @@ class PublisherTestCase(unittest.TestCase):
 
         request = self.new_request_for(result_view)
         publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        result = publication(lambda: request.close())
 
         self.assertEqual(
             request.mocker_called(),
@@ -407,7 +429,10 @@ class PublisherTestCase(unittest.TestCase):
             [('Content-Type', 'text/html;charset=utf-8')])
         self.assertEqual(
             get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])
+            ['PublicationStart',
+             'PublicationAfterTraversal',
+             'PublicationAfterRender',
+             'PublicationBeforeStreaming'])
 
         self.assertRaises(ConflictError, consume_wsgi_result, result)
 
@@ -418,8 +443,12 @@ class PublisherTestCase(unittest.TestCase):
             [('commit', (), {}),  ('abort', (), {})])
         self.assertEqual(
             get_event_names(),
-            ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
-             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure'])
+            ['TestNextCalled',
+             'TestNextCalled',
+             'TestNextCalled',
+             'PublicationBeforeCommit',
+             'PublicationBeforeAbort',
+             'PublicationFailure'])
 
     def test_streamiterator(self):
         """Test a view that return an object of type
@@ -429,7 +458,7 @@ class PublisherTestCase(unittest.TestCase):
         """
         request = self.new_request_for(streamiterator_view)
         publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        result = publication(lambda: request.close())
 
         self.assertEqual(
             request.mocker_called(),
@@ -446,7 +475,9 @@ class PublisherTestCase(unittest.TestCase):
              ('Content-Type', 'text/html;charset=utf-8')])
         self.assertEqual(
             get_event_names(),
-            ['PubStart', 'PubAfterTraversal'])
+            ['PublicationStart',
+             'PublicationAfterTraversal',
+             'PublicationAfterRender'])
 
         body = consume_wsgi_result(result)
 
@@ -457,8 +488,11 @@ class PublisherTestCase(unittest.TestCase):
             self.app.transaction.mocker_called(), [('commit', (), {})])
         self.assertEqual(
             get_event_names(),
-            ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
-             'PubBeforeCommit', 'PubSuccess'])
+            ['TestNextCalled',
+             'TestNextCalled',
+             'TestNextCalled',
+             'PublicationBeforeCommit',
+             'PublicationSuccess'])
 
     def test_bugous_streamiterator(self):
         """Test a view that return an object of type IStreamIterator,
@@ -467,7 +501,7 @@ class PublisherTestCase(unittest.TestCase):
         """
         request = self.new_request_for(bugous_streamiterator_view)
         publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        result = publication(lambda: request.close())
 
         self.assertEqual(
             request.mocker_called(),
@@ -484,7 +518,9 @@ class PublisherTestCase(unittest.TestCase):
              ('Content-Type', 'text/html;charset=utf-8')])
         self.assertEqual(
             get_event_names(),
-            ['PubStart', 'PubAfterTraversal'])
+            ['PublicationStart',
+             'PublicationAfterTraversal',
+             'PublicationAfterRender'])
 
         self.assertRaises(ValueError, consume_wsgi_result, result)
 
@@ -494,7 +530,9 @@ class PublisherTestCase(unittest.TestCase):
             self.app.transaction.mocker_called(), [('abort', (), {})])
         self.assertEqual(
             get_event_names(),
-            ['TestNextCalled', 'PubBeforeAbort', 'PubFailure'])
+            ['TestNextCalled',
+             'PublicationBeforeAbort',
+             'PublicationFailure'])
 
     def test_streamiterator_with_conflict_error(self):
         """Test a view that return an object of type IStreamIterator,
@@ -506,7 +544,7 @@ class PublisherTestCase(unittest.TestCase):
 
         request = self.new_request_for(streamiterator_view)
         publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        result = publication(lambda: request.close())
 
         self.assertEqual(
             request.mocker_called(),
@@ -523,7 +561,9 @@ class PublisherTestCase(unittest.TestCase):
              ('Content-Type', 'text/html;charset=utf-8')])
         self.assertEqual(
             get_event_names(),
-            ['PubStart', 'PubAfterTraversal'])
+            ['PublicationStart',
+             'PublicationAfterTraversal',
+             'PublicationAfterRender'])
 
         self.assertRaises(ConflictError, consume_wsgi_result, result)
 
@@ -534,8 +574,12 @@ class PublisherTestCase(unittest.TestCase):
             [('commit', (), {}), ('abort', (), {})])
         self.assertEqual(
             get_event_names(),
-            ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
-             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure'])
+            ['TestNextCalled',
+             'TestNextCalled',
+             'TestNextCalled',
+             'PublicationBeforeCommit',
+             'PublicationBeforeAbort',
+             'PublicationFailure'])
 
     def test_bugous_view(self):
         """Test a broken view.
@@ -543,7 +587,7 @@ class PublisherTestCase(unittest.TestCase):
         request = self.new_request_for(bugous_view)
         with LoggingTesting('infrae.wsgi') as logs:
             publication = WSGIPublication(self.app, request, self.response)
-            result = publication()
+            result = publication(lambda: request.close())
 
             self.assertEqual(
                 request.mocker_called(),
@@ -561,11 +605,15 @@ class PublisherTestCase(unittest.TestCase):
                 ('Content-Type', 'text/html;charset=utf-8')])
             self.assertEqual(
                 get_event_names(),
-                ['PubStart', 'PubAfterTraversal', 'PubBeforeAbort', 'PubFailure'])
+                ['PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationAfterRender',
+                 'PublicationBeforeAbort',
+                 'PublicationFailure'])
 
             body = consume_wsgi_result(result)
 
-            self.failUnless('I am not happy' in body)
+            self.assertTrue('I am not happy' in body)
             self.assertEqual(
                 request.mocker_called(),  [('close', (), {})])
             self.assertEqual(
@@ -581,7 +629,7 @@ class PublisherTestCase(unittest.TestCase):
         request = self.new_request_for(invalid_view)
         with LoggingTesting('infrae.wsgi') as logs:
             publication = WSGIPublication(self.app, request, self.response)
-            result = publication()
+            result = publication(lambda: request.close())
 
             self.assertEqual(
                 request.mocker_called(),
@@ -598,7 +646,11 @@ class PublisherTestCase(unittest.TestCase):
                [])
             self.assertEqual(
                 get_event_names(),
-                ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+                ['PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationAfterRender',
+                 'PublicationBeforeCommit',
+                 'PublicationSuccess'])
 
             consume_wsgi_result(result)
 
@@ -618,7 +670,7 @@ class PublisherTestCase(unittest.TestCase):
         """
         request = self.new_request_for(not_found_view)
         publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        result = publication(lambda: request.close())
 
         self.assertEqual(
             request.mocker_called(),
@@ -635,11 +687,15 @@ class PublisherTestCase(unittest.TestCase):
              ('Content-Type', 'text/html;charset=utf-8')])
         self.assertEqual(
             get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeAbort', 'PubFailure'])
+            ['PublicationStart',
+             'PublicationAfterTraversal',
+             'PublicationAfterRender',
+             'PublicationBeforeAbort',
+             'PublicationFailure'])
 
         body = consume_wsgi_result(result)
 
-        self.failUnless('Page not found' in body)
+        self.assertTrue('Page not found' in body)
         self.assertEqual(
             request.mocker_called(),  [('close', (), {})])
         self.assertEqual(
@@ -652,7 +708,7 @@ class PublisherTestCase(unittest.TestCase):
         """
         request = self.new_request_for(redirect_view)
         publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        result = publication(lambda: request.close())
 
         self.assertEqual(
             request.mocker_called(),
@@ -669,7 +725,10 @@ class PublisherTestCase(unittest.TestCase):
              ('Location', 'http://infrae.com/products/silva')])
         self.assertEqual(
             get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+            ['PublicationStart',
+             'PublicationAfterTraversal',
+             'PublicationBeforeCommit',
+             'PublicationSuccess'])
 
         body = consume_wsgi_result(result)
 
@@ -686,7 +745,7 @@ class PublisherTestCase(unittest.TestCase):
         """
         request = self.new_request_for(unauthorized_view)
         publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        result = publication(lambda: request.close())
 
         self.assertEqual(
             request.mocker_called(),
@@ -703,7 +762,10 @@ class PublisherTestCase(unittest.TestCase):
              ('Www-Authenticate', 'basic realm="Zope"')])
         self.assertEqual(
             get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+            ['PublicationStart',
+             'PublicationAfterTraversal',
+             'PublicationBeforeCommit',
+             'PublicationSuccess'])
 
         body = consume_wsgi_result(result)
 
@@ -723,7 +785,8 @@ class PublisherTestCase(unittest.TestCase):
             response = WSGIResponse({}, self.app.response)
             response._unauthorized = bugous_view # Set the bugous handler
             publication = WSGIPublication(self.app, request, response)
-            result = publication()
+            result = publication(lambda: request.close())
+
             self.assertEqual(
                 request.mocker_called(),
                 [('processInputs', (), {})])
@@ -739,7 +802,10 @@ class PublisherTestCase(unittest.TestCase):
                  ('Www-Authenticate', 'basic realm="Zope"')])
             self.assertEqual(
                 get_event_names(),
-                ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+                ['PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationBeforeCommit',
+                 'PublicationSuccess'])
 
             body = consume_wsgi_result(result)
 
@@ -751,14 +817,15 @@ class PublisherTestCase(unittest.TestCase):
             self.assertEqual(
                 get_event_names(), [])
 
-            logs.assertContains('Error while processing the unauthorized PAS handler')
+            logs.assertContains(
+                'Error while processing the unauthorized PAS handler')
 
     def test_forbidden(self):
         """Test a view which raises the Forbidden exception.
         """
         request = self.new_request_for(forbidden_view)
         publication = WSGIPublication(self.app, request, self.response)
-        result = publication()
+        result = publication(lambda: request.close())
 
         self.assertEqual(
             request.mocker_called(),
@@ -775,11 +842,15 @@ class PublisherTestCase(unittest.TestCase):
              ('Content-Type', 'text/html;charset=utf-8')])
         self.assertEqual(
             get_event_names(),
-            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+            ['PublicationStart',
+             'PublicationAfterTraversal',
+             'PublicationAfterRender',
+             'PublicationBeforeCommit',
+             'PublicationSuccess'])
 
         body = consume_wsgi_result(result)
 
-        self.failUnless('Go away' in body)
+        self.assertTrue('Go away' in body)
         self.assertEqual(
             request.mocker_called(),  [('close', (), {})])
         self.assertEqual(
@@ -796,7 +867,7 @@ class PublisherTestCase(unittest.TestCase):
         with LoggingTesting('infrae.wsgi') as logs:
 
             publication = WSGIPublication(self.app, request, self.response)
-            body = consume_wsgi_result(publication())
+            body = consume_wsgi_result(publication(lambda: request.close()))
 
             self.assertEqual(
                 self.app.transaction.mocker_called(),
@@ -812,25 +883,82 @@ class PublisherTestCase(unittest.TestCase):
                  ('recordMetaData', (hello_view, request), {}),
                  ('commit', (), {}),
                  ('abort', (), {})])
-            self.assertEqual(self.app.response.status, '503 Service Unavailable')
+            self.assertEqual(
+                self.app.response.status,
+                '503 Service Unavailable')
             self.assertEqual(
                 self.app.response.headers,
                 [('Content-Length', '198'),
                  ('Content-Type', 'text/html;charset=utf-8')])
-            self.failUnless('Service temporarily unavailable' in body)
+            self.assertTrue('Service temporarily unavailable' in body)
             self.assertEqual(
                 get_event_names(),
-                ['PubStart', 'PubAfterTraversal',
-                 'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',
-                 'PubStart', 'PubAfterTraversal',
-                 'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',
-                 'PubStart', 'PubAfterTraversal',
-                 'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',])
+                ['PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationAfterRender',
+                 'PublicationBeforeCommit',
+                 'PublicationBeforeAbort',
+                 'PublicationFailure',
+                 'PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationAfterRender',
+                 'PublicationBeforeCommit',
+                 'PublicationBeforeAbort',
+                 'PublicationFailure',
+                 'PublicationStart',
+                 'PublicationAfterTraversal',
+                 'PublicationAfterRender',
+                 'PublicationBeforeCommit',
+                 'PublicationBeforeAbort',
+                 'PublicationFailure',])
 
             logs.assertContains(
                 'Conflict error for request http://infrae.com/index.html')
 
-    def test_conflict_error_but_ok(self):
+    def test_conflict_errors_but_ok(self):
+        """Test a view which works after triggering one conflict errors, itself.
+        """
+        self.app.transaction.mocker_set_conflict(1)
+
+        request = self.new_request_for(hello_view)
+
+        publication = WSGIPublication(self.app, request, self.response)
+        body = consume_wsgi_result(publication(lambda: request.close()))
+
+        self.assertEqual(
+            request.mocker_called(),
+            [('processInputs', (), {}), ('retry', (), {}),
+             ('processInputs', (), {}), ('close', (), {}), ('close', (), {})])
+        self.assertEqual(
+            self.app.transaction.mocker_called(),
+            [('begin', (), {}),
+             ('recordMetaData', (hello_view, request), {}),
+             ('commit', (), {}),
+             ('abort', (), {}),
+             ('begin', (), {}),
+             ('recordMetaData', (hello_view, request), {}),
+             ('commit', (), {})])
+        self.assertEqual(self.app.response.status, '200 OK')
+        self.assertEqual(
+            self.app.response.headers,
+            [('Content-Length', '12'),
+             ('Content-Type', 'text/html;charset=utf-8')])
+        self.assertEqual(body, 'Hello world!')
+        self.assertEqual(
+            get_event_names(),
+             ['PublicationStart',
+              'PublicationAfterTraversal',
+              'PublicationAfterRender',
+              'PublicationBeforeCommit',
+              'PublicationBeforeAbort',
+              'PublicationFailure',
+              'PublicationStart',
+              'PublicationAfterTraversal',
+              'PublicationAfterRender',
+              'PublicationBeforeCommit',
+              'PublicationSuccess'])
+
+    def test_conflict_errors_on_view_but_ok(self):
         """Test a view which works after triggering one conflict errors, itself.
         """
         global conflict_count
@@ -838,7 +966,7 @@ class PublisherTestCase(unittest.TestCase):
 
         request = self.new_request_for(not_so_conflictuous_view)
         publication = WSGIPublication(self.app, request, self.response)
-        body = consume_wsgi_result(publication())
+        body = consume_wsgi_result(publication(lambda: request.close()))
 
         self.assertEqual(
             request.mocker_called(),
@@ -861,8 +989,15 @@ class PublisherTestCase(unittest.TestCase):
         self.assertEqual(body, 'I worked fine')
         self.assertEqual(
             get_event_names(),
-             ['PubStart', 'PubAfterTraversal', 'PubBeforeAbort', 'PubFailure',
-              'PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])
+             ['PublicationStart',
+              'PublicationAfterTraversal',
+              'PublicationBeforeAbort',
+              'PublicationFailure',
+              'PublicationStart',
+              'PublicationAfterTraversal',
+              'PublicationAfterRender',
+              'PublicationBeforeCommit',
+              'PublicationSuccess'])
 
 
 def test_suite():
